unit md5;

interface

uses SysUtils;

const MD5_CBLOCK = 64;
const MD5_LBLOCK = (MD5_CBLOCK div 4);
const MD5_DIGEST_LENGTH = 16;

type
   MD5_CTX = record
      A, B, C, D : LongWord;
      Ml, Mh : LongWord;
      data : array[1..MD5_LBLOCK] of LongWord;
      num : Integer;
   end;

   PMD5_CTX = ^MD5_CTX;


procedure MD5_Init(c : PMD5_CTX); cdecl;
procedure MD5_Update(c : PMD5_CTX; Buffer : PChar; Len : LongWord); cdecl;
procedure MD5_Final(md : PChar; c : PMD5_CTX); cdecl;

function MD5Hash(FileName : String) : String;

type
   TMD5Hash = class
   public
      constructor Create;
      destructor Destroy; override;

      procedure Update(Buffer : PChar; Len : Integer);
      function Final : String;

   private
      Ctx        : MD5_CTX;
      Finalised  : Boolean;
      FinalValue : String;
   end;

implementation

uses WinBinFile;

function MD5Hash(FileName : String) : String;
var
   Ctx     : MD5_CTX;
   BinFile : TBinaryFile;
   Buffer  : String;
   Actual  : Integer;
   i : Integer;
begin
   BinFile := TBinaryFile.Create;
   try
      BinFile.Assign(FileName);
      BinFile.Open(OPEN_READ_ONLY);

      MD5_Init(@Ctx);

      SetLength(Buffer, 1024);
      while true do
      begin
         Actual := BinFile.BlockRead2(PChar(Buffer), 1024);
         if Actual = 0 then
         begin
            break;
         end;

         MD5_Update(@Ctx, PChar(Buffer), Actual);
      end;

      SetLength(Buffer, MD5_DIGEST_LENGTH);
      MD5_Final(PChar(Buffer), @Ctx);

      for i := 1 to MD5_DIGEST_LENGTH do
      begin
         Result := Result + IntToHex(Integer(Buffer[i]), 2);
      end;
   finally
      BinFile.Free;
   end;
end;

constructor TMD5Hash.Create;
begin
   inherited Create;

   MD5_Init(@Ctx);
   Finalised := False;
end;

destructor TMD5Hash.Destroy;
begin
   inherited Destroy;
end;

procedure TMD5Hash.Update(Buffer : PChar; Len : Integer);
begin
   if not Finalised then
   begin
      MD5_Update(@Ctx, Buffer, Len);
   end;
end;

function TMD5Hash.Final : String;
var
   Buffer  : String;
   i : Integer;
begin
   if not Finalised then
   begin
      SetLength(Buffer, MD5_DIGEST_LENGTH);
      MD5_Final(PChar(Buffer), @Ctx);

      FinalValue := '';
      for i := 1 to MD5_DIGEST_LENGTH do
      begin
         FinalValue := FinalValue + IntToHex(Integer(Buffer[i]), 2);
      end;

      Finalised := True;
   end;
   Result := FinalValue;
end;


const DLLNAME = 'ssl.dll';

procedure MD5_Init; external DLLNAME;
procedure MD5_Update; external DLLNAME;
procedure MD5_Final; external DLLNAME;


end.
