/*
	$Header: /home/itig/cvsroot/ext2ifs/src/superblock.c,v 1.7 2001/11/11 09:52:27 jn Exp $
*/

#include "ext2fs.h"


/*
	This just verifies that the device contains an ext2 fs.
*/
NTSTATUS
XtSBCheckDeviceForExt2fs (
	IN PDEVICE_OBJECT DeviceObject
	)
{
	PUCHAR			buffer;
	LARGE_INTEGER	offset;
	NTSTATUS		status;
	USHORT			magic;

	int i;
	int j;
	int o = 0;

	//PAGED_CODE();
	ASSERT(DeviceObject != NULL);

	buffer = (PUCHAR) ExAllocatePool(NonPagedPoolCacheAligned, 512);

	if (!buffer)
	{
		return STATUS_INSUFFICIENT_RESOURCES;
	}

	offset.QuadPart = EXT2_SUPER_OFFSET;

	status = XtBLReadBlockDevice(
		DeviceObject,
		&offset,
		512,
		buffer
		);

	magic = *( (PUSHORT) &buffer[EXT2_SUPER_MAGIC_OFFSET] );

	ExFreePool(buffer);

	if (!NT_SUCCESS(status))
	{
		KdPrint(("[EXT2]: CheckDeviceForExt2fs: "
			"ReadBlockDevice failed with error 0x%x\n", status));
		return status;
	}
	else if (magic == EXT2_SUPER_MAGIC)
	{
		KdPrint(("[EXT2]: CheckDeviceForExt2fs: Ext2fs found\n"));
		return STATUS_SUCCESS;//FS_DRIVER_REQUIRED;
	}
	else
	{
//		KdPrint(("[EXT2]: CheckDeviceForExt2fs: Ext2fs not found\n"));
		return STATUS_UNRECOGNIZED_VOLUME;
	}
}



/*
	This will load in the superblock and init all the low level IO for the device
*/
NTSTATUS 
XtSBMountFS(
	PtrXtFSVCB Vcb
)
{
	NTSTATUS Status;
	LARGE_INTEGER	Offset;
	PVOID           SuperBlockBcb;
	PSUPER_BLOCK	SuperBlock;

	//PAGED_CODE();


	Offset.QuadPart = EXT2_SUPER_OFFSET;

	ACQUIRE_EX(Vcb->Resource);
	ASSERT(Vcb);
	ASSERT(Vcb->VirtualVolumeFile);

	
	KdPrint(("[EXT2]: About to map superblock\n"));
	Status = STATUS_SUCCESS;

	if(CcPinRead(Vcb->VirtualVolumeFile, &Offset, sizeof(SUPER_BLOCK), TRUE, &SuperBlockBcb, &SuperBlock))
	{
		KdPrint(("[EXT2]: Map done\n"));
		if(SuperBlock->s_magic == EXT2_SUPER_MAGIC)
		{
			KdPrint(("[EXT2]: Magic is still good\n"));
			Vcb->SuperBlock = SuperBlock;
			XtBLInitBlockIO(Vcb);
		}
		else
		{
			KdPrint(("[EXT2]: Magic is 0x%04x\n", SuperBlock->s_magic));
			Status = STATUS_UNRECOGNIZED_VOLUME;
		}	
	}
	else
	{
		KdPrint(("[EXT2]: Map failed\n"));
	}

	RELEASE(Vcb->Resource);


	return Status;
}
